"""Client for django-secretary."""
import json
from typing import Callable, Tuple, Union
from urllib.parse import urljoin
from uuid import UUID

import requests
from requests import HTTPError, RequestException, Response


class TemplateNotFound(ValueError):
    """Template not found exception."""


def process_errors(func: Callable[..., Response]) -> Callable[..., Response]:
    """
    Process commonly raised connection errors.

    Args:
        func: Function returning the requests Response object.

    Raises:
        TemplateNotFound: If response status code is 404.
        ValueError: In case of any other HTTP error (such as 403 or 500).
        RuntimeError: In case of any other error (such as ConnectionError).
    """
    def wrapper(*args, **kwargs) -> Response:
        try:
            response = func(*args, **kwargs)
            if response.status_code == 404:
                raise TemplateNotFound('Template not found at url: {}'.format(response.request.url))
            response.raise_for_status()
        except HTTPError as error:
            if response.status_code >= 400 and response.status_code < 500:
                raise ValueError(str(error))
            else:
                raise RuntimeError(str(error))
        except RequestException as error:
            raise RuntimeError(str(error))

        return response

    return wrapper


class SecretaryClient:
    """Client for django-secretary."""

    def __init__(self, server_url: str, timeout: Union[float, Tuple[float, float]] = 3.05):
        """
        Initialize REST API client.

        Args:
            server_url: Server url of django-secretary app, including the path to the REST API with trailing slash.
            timeout: Timeout for the django-secretary API.
        """
        self.server_url = server_url
        self.timeout = timeout

    def render(self, template: Union[UUID, str], context: dict) -> str:
        """
        Render plaintext template.

        Args:
            template: Name of template or UUID.
            context: Context passed to template for rendering.

        Returns:
            Rendered plain text template.

        Raises:
            TemplateNotFound: If response status code is 404.
            ValueError: In case of any other HTTP error (such as 403 or 500).
            RuntimeError: In case of any other error (such as ConnectionError).
        """
        response = self._get_render_response(template, context, accept='text/plain')
        return response.text

    def render_html(self, template: Union[UUID, str], context: dict) -> str:
        """
        Render HTML template.

        Args:
            template: Name of template or UUID.
            context: Context passed to template for rendering.

        Returns:
            Rendered HTML template.

        Raises:
            TemplateNotFound: If response status code is 404.
            ValueError: In case of any other HTTP error (such as 403 or 500).
            RuntimeError: In case of any other error (such as ConnectionError).
        """
        response = self._get_render_response(template, context, accept='text/html')
        return response.text

    def get_template_info(self, template: Union[UUID, str]) -> dict:
        """
        Get template info.

        Args:
            template: Name of template or UUID.

        Returns:
            Template info (name, uuid, is_active, assets, ...).

        Raises:
            TemplateNotFound: If response status code is 404.
            ValueError: In case of any other HTTP error (such as 403 or 500).
            RuntimeError: In case of any other error (such as ConnectionError).
        """
        response = process_errors(requests.get)(
            self._get_template_url(template),
            timeout=self.timeout,
        )

        return response.json()

    def _get_template_url(self, template: Union[UUID, str]) -> str:
        """Get template url."""
        if isinstance(template, UUID):
            return urljoin(self.server_url, 'templates/{}/'.format(template))
        else:
            return urljoin(self.server_url, 'templates/active/{}/'.format(template))

    @process_errors
    def _get_render_response(self, template: Union[UUID, str], context: dict, *, accept: str) -> Response:
        """Render template."""
        response = requests.post(
            urljoin(self._get_template_url(template), 'render/'),
            headers={'Content-Type': 'application/json', 'Accept': accept},
            data=json.dumps({'context': context}),
            timeout=self.timeout,
        )

        return response
