from unittest import TestCase
from uuid import UUID

import requests
import responses

from typist.client import SecretaryClient, TemplateNotFound, process_errors


class TestSecretaryClient(TestCase):

    def setUp(self):
        self.client = SecretaryClient('http://example.com/api/')
        self.uuid = UUID(int=42)
        self.name = 'template.txt'

    @responses.activate
    def test_render_by_uuid(self):
        url = 'http://example.com/api/templates/{}/render/'.format(self.uuid)
        responses.add(responses.POST, url, body=b'rendered template')
        self.assertEqual(self.client.render(self.uuid, context={}), 'rendered template')
        responses.assert_call_count(url, 1)
        self.assertEqual(responses.calls[0].request.headers['Accept'], 'text/plain')

    @responses.activate
    def test_render_by_name(self):
        url = 'http://example.com/api/templates/active/{}/render/'.format(self.name)
        responses.add(responses.POST, url, body=b'rendered template')
        self.assertEqual(self.client.render(self.name, context={}), 'rendered template')
        responses.assert_call_count(url, 1)
        self.assertEqual(responses.calls[0].request.headers['Accept'], 'text/plain')

    @responses.activate
    def test_render_html_by_uuid(self):
        url = 'http://example.com/api/templates/{}/render/'.format(self.uuid)
        responses.add(responses.POST, url, body=b'<b>rendered</b> template')
        self.assertEqual(self.client.render_html(self.uuid, context={}), '<b>rendered</b> template')
        responses.assert_call_count(url, 1)
        self.assertEqual(responses.calls[0].request.headers['Accept'], 'text/html')

    @responses.activate
    def test_render_html_by_name(self):
        url = 'http://example.com/api/templates/active/{}/render/'.format(self.name)
        responses.add(responses.POST, url, body=b'<b>rendered</b> template')
        self.assertEqual(self.client.render_html(self.name, context={}), '<b>rendered</b> template')
        responses.assert_call_count(url, 1)
        self.assertEqual(responses.calls[0].request.headers['Accept'], 'text/html')

    @responses.activate
    def test_get_template_info_by_uuid(self):
        url = 'http://example.com/api/templates/{}/'.format(self.uuid)
        responses.add(responses.GET, url, json={'name': self.name})
        self.assertEqual(self.client.get_template_info(self.uuid), {'name': self.name})
        responses.assert_call_count(url, 1)

    @responses.activate
    def test_get_template_info_by_name(self):
        url = 'http://example.com/api/templates/active/{}/'.format(self.name)
        responses.add(responses.GET, url, json={'name': self.name})
        self.assertEqual(self.client.get_template_info(self.name), {'name': self.name})
        responses.assert_call_count(url, 1)


class TestProcessErrors(TestCase):

    @responses.activate
    def test_no_error(self):
        responses.add(responses.GET, 'http://example.com/')
        response = process_errors(requests.get)('http://example.com')
        self.assertTrue(response.ok)

    @responses.activate
    def test_not_found(self):
        responses.add(responses.GET, 'http://example.com/', status=404)
        with self.assertRaisesRegex(TemplateNotFound, 'Template not found at url: http://example.com/'):
            process_errors(requests.get)('http://example.com')

    @responses.activate
    def test_forbidden(self):
        responses.add(responses.GET, 'http://example.com/', status=403)
        with self.assertRaisesRegex(ValueError, '403 Client Error: Forbidden for url: http://example.com/'):
            process_errors(requests.get)('http://example.com')

    @responses.activate
    def test_internal_server_error(self):
        responses.add(responses.GET, 'http://example.com/', status=500)
        with self.assertRaisesRegex(
            RuntimeError,
            '500 Server Error: Internal Server Error for url: http://example.com/'
        ):
            process_errors(requests.get)('http://example.com')

    @responses.activate
    def test_connection_error(self):
        responses.add(responses.GET, 'http://example.com/', body=requests.ConnectionError('Failed to connect'))
        with self.assertRaisesRegex(RuntimeError, '^Failed to connect$'):
            process_errors(requests.get)('http://example.com')
